@@ -1,10 +1,7 @@
 #pragma once
 
-#include <stdint.h>
-
-void initAboutScreen(void);
 void renderAboutScreenFrame(void);
-void seedAboutScreenRandom(uint32_t newseed);
+void initAboutScreen(void);
 void showAboutScreen(void);
 void hideAboutScreen(void);
 void exitAboutScreen(void);
