@@ -14,6 +14,7 @@
 #include "ft2_wav_renderer.h"
 #include "ft2_tables.h"
 #include "ft2_structs.h"
+#include "ft2_audioselector.h"
 #include "mixer/ft2_mix.h"
 #include "mixer/ft2_silence_mix.h"
 
@@ -36,17 +37,6 @@ chSync_t chSync;
 pattSync_t pattSync;
 volatile bool pattQueueClearing, chQueueClearing;
 
-void resetCachedMixerVars(void)
-{
-	channel_t *ch = channel;
-	for (int32_t i = 0; i < MAX_CHANNELS; i++, ch++)
-		ch->oldFinalPeriod = -1;
-
-	voice_t *v = voice;
-	for (int32_t i = 0; i < MAX_CHANNELS*2; i++, v++)
-		v->oldDelta = 0;
-}
-
 void stopVoice(int32_t i)
 {
 	voice_t *v;
@@ -177,6 +167,7 @@ void setMixerBPM(int32_t bpm)
 
 	audio.samplesPerTickInt = audio.samplesPerTickIntTab[i];
 	audio.samplesPerTickFrac = audio.samplesPerTickFracTab[i];
+	audio.fSamplesPerTickIntMul = (float)(1.0 / (double)audio.samplesPerTickInt);
 
 	// for audio/video sync timestamp
 	tickTimeLenInt = audio.tickTimeIntTab[i];
@@ -200,17 +191,17 @@ void audioSetInterpolationType(uint8_t interpolationType)
 	// set sinc LUT pointers
 	if (config.interpolation == INTERPOLATION_SINC8)
 	{
-		fKaiserSinc = fKaiserSinc_8;
-		fDownSample1 = fDownSample1_8;
-		fDownSample2 = fDownSample2_8;
+		fSinc_1 = fSinc8_1;
+		fSinc_2 = fSinc8_2;
+		fSinc_3 = fSinc8_3;
 
 		audio.sincInterpolation = true;
 	}
 	else if (config.interpolation == INTERPOLATION_SINC16)
 	{
-		fKaiserSinc = fKaiserSinc_16;
-		fDownSample1 = fDownSample1_16;
-		fDownSample2 = fDownSample2_16;
+		fSinc_1 = fSinc16_1;
+		fSinc_2 = fSinc16_2;
+		fSinc_3 = fSinc16_3;
 
 		audio.sincInterpolation = true;
 	}
@@ -261,10 +252,8 @@ static void voiceUpdateVolumes(int32_t i, uint8_t status)
 			const float fVolumeRDiff = 0.0f - f->fCurrVolumeR;
 
 			f->volumeRampLength = audio.quickVolRampSamples; // 5ms
-			const float fVolumeRampLength = (float)(int32_t)f->volumeRampLength;
-
-			f->fVolumeLDelta = fVolumeLDiff / fVolumeRampLength;
-			f->fVolumeRDelta = fVolumeRDiff / fVolumeRampLength;
+			f->fVolumeLDelta = fVolumeLDiff * audio.fQuickVolRampSamplesMul;
+			f->fVolumeRDelta = fVolumeRDiff * audio.fQuickVolRampSamplesMul;
 
 			f->isFadeOutVoice = true;
 		}
@@ -282,12 +271,20 @@ static void voiceUpdateVolumes(int32_t i, uint8_t status)
 		const float fVolumeLDiff = v->fTargetVolumeL - v->fCurrVolumeL;
 		const float fVolumeRDiff = v->fTargetVolumeR - v->fCurrVolumeR;
 
-		// IS_QuickVol = 5ms, otherwise the duration of a tick
-		v->volumeRampLength = (status & IS_QuickVol) ? audio.quickVolRampSamples : audio.samplesPerTickInt;
-		const float fVolumeRampLength = (float)(int32_t)v->volumeRampLength;
+		float fRampLengthMul;
+		if (status & IS_QuickVol) // duration of 5ms
+		{
+			v->volumeRampLength = audio.quickVolRampSamples;
+			fRampLengthMul = audio.fQuickVolRampSamplesMul;
+		}
+		else // duration of a tick
+		{
+			v->volumeRampLength = audio.samplesPerTickInt;
+			fRampLengthMul = audio.fSamplesPerTickIntMul;
+		}
 
-		v->fVolumeLDelta = fVolumeLDiff / fVolumeRampLength;
-		v->fVolumeRDelta = fVolumeRDiff / fVolumeRampLength;
+		v->fVolumeLDelta = fVolumeLDiff * fRampLengthMul;
+		v->fVolumeRDelta = fVolumeRDiff * fRampLengthMul;
 	}
 }
 
@@ -324,7 +321,7 @@ static void voiceTrigger(int32_t ch, sample_t *s, int32_t position)
 		v->leftEdgeTaps8 = s->leftEdgeTapSamples8 + MAX_LEFT_TAPS;
 	}
 
-	v->hasLooped = false; // for sinc interpolation special case
+	v->hasLooped = false; // for cubic/sinc interpolation special case
 	v->samplingBackwards = false;
 	v->loopType = loopType;
 	v->sampleEnd = (loopType == LOOP_OFF) ? length : loopEnd;
@@ -340,7 +337,7 @@ static void voiceTrigger(int32_t ch, sample_t *s, int32_t position)
 		return;
 	}
 
-	v->mixFuncOffset = ((int32_t)sample16Bit * 15) + (audio.interpolationType * 3) + loopType;
+	v->mixFuncOffset = ((int32_t)sample16Bit * 18) + (audio.interpolationType * 3) + loopType;
 	v->active = true;
 }
 
@@ -370,11 +367,8 @@ void updateVoices(void)
 
 		if (status & IS_Vol)
 		{
-			v->fVolume = ch->fFinalVol;
-
-			// set scope volume
-			const int32_t scopeVolume = (int32_t)((SCOPE_HEIGHT * ch->fFinalVol) + 0.5f); // rounded
-			v->scopeVolume = (uint8_t)scopeVolume;
+			v->fVolume = ch->fFinalVol; // 0.0f .. 1.0f
+			v->scopeVolume = (uint8_t)((ch->fFinalVol * 255.0f) + 0.5f); // 0..255, rounded
 		}
 
 		if (status & IS_Pan)
@@ -385,32 +379,20 @@ void updateVoices(void)
 
 		if (status & IS_Period)
 		{
-			// use cached values when possible
-			if (ch->finalPeriod != ch->oldFinalPeriod)
+			const double dVoiceHz = dPeriod2Hz(ch->finalPeriod);
+
+			// set voice delta
+			v->delta = (int64_t)((dVoiceHz * audio.dHz2MixDeltaMul) + 0.5); // Hz -> fixed-point delta (rounded)
+			if (audio.sincInterpolation)
 			{
-				ch->oldFinalPeriod = ch->finalPeriod;
-
-				const double dHz = dPeriod2Hz(ch->finalPeriod);
-
-				// set voice delta
-				const uint64_t delta = v->oldDelta = (int64_t)((dHz * audio.dHz2MixDeltaMul) + 0.5); // Hz -> fixed-point delta (rounded)
-
-				if (audio.sincInterpolation) // decide which sinc LUT to use according to the resampling ratio
-				{
-					if (delta <= sincDownsample1Ratio)
-						v->fSincLUT = fKaiserSinc;
-					else if (delta <= sincDownsample2Ratio)
-						v->fSincLUT = fDownSample1;
-					else
-						v->fSincLUT = fDownSample2;
-				}
-
-				// set scope delta
-				const double dHz2ScopeDeltaMul = SCOPE_FRAC_SCALE / (double)SCOPE_HZ;
-				v->scopeDelta = (int64_t)((dHz * dHz2ScopeDeltaMul) + 0.5); // Hz -> fixed-point delta (rounded)
+				// decide which sinc LUT to use according to the resampling ratio
+				if (v->delta <= sincRatio1)
+					v->fSincLUT = fSinc_1;
+				else if (v->delta <= sincRatio2)
+					v->fSincLUT = fSinc_2;
+				else
+					v->fSincLUT = fSinc_3;
 			}
-
-			v->delta = v->oldDelta;
 		}
 
 		if (status & IS_Trigger)
@@ -423,8 +405,6 @@ static void sendSamples16BitStereo(void *stream, uint32_t sampleBlockLength)
 	int16_t *streamPtr16 = (int16_t *)stream;
 	for (uint32_t i = 0; i < sampleBlockLength; i++)
 	{
-		// TODO: This could use dithering (a proper implementation, that is...)
-
 		int32_t L = (int32_t)(audio.fMixBufferL[i] * fAudioNormalizeMul);
 		int32_t R = (int32_t)(audio.fMixBufferR[i] * fAudioNormalizeMul);
 
@@ -435,8 +415,7 @@ static void sendSamples16BitStereo(void *stream, uint32_t sampleBlockLength)
 		*streamPtr16++ = (int16_t)R;
 
 		// clear what we read from the mixing buffer
-		audio.fMixBufferL[i] = 0.0f;
-		audio.fMixBufferR[i] = 0.0f;
+		audio.fMixBufferL[i] = audio.fMixBufferR[i] = 0.0f;
 	}
 }
 
@@ -452,8 +431,7 @@ static void sendSamples32BitFloatStereo(void *stream, uint32_t sampleBlockLength
 		*fStreamPtr32++ = CLAMP(fR, -1.0f, 1.0f);
 
 		// clear what we read from the mixing buffer
-		audio.fMixBufferL[i] = 0.0f;
-		audio.fMixBufferR[i] = 0.0f;
+		audio.fMixBufferL[i] = audio.fMixBufferR[i] = 0.0f;
 	}
 }
 
@@ -462,6 +440,8 @@ static void doChannelMixing(int32_t bufferPosition, int32_t samplesToMix)
 	voice_t *v = voice; // normal voices
 	voice_t *r = &voice[MAX_CHANNELS]; // volume ramp fadeout-voices
 
+	const int32_t mixOffsetBias = 3 * NUM_INTERPOLATORS * 2; // 3 = loop types (off/fwd/bidi), 2 = bit depths (8-bit/16-bit)
+
 	for (int32_t i = 0; i < song.numChannels; i++, v++, r++)
 	{
 		if (v->active)
@@ -470,11 +450,11 @@ static void doChannelMixing(int32_t bufferPosition, int32_t samplesToMix)
 			if (!volRampFlag && v->fCurrVolumeL == 0.0f && v->fCurrVolumeR == 0.0f)
 				silenceMixRoutine(v, samplesToMix);
 			else
-				mixFuncTab[((int32_t)volRampFlag * (3*5*2)) + v->mixFuncOffset](v, bufferPosition, samplesToMix);
+				mixFuncTab[((int32_t)volRampFlag * mixOffsetBias) + v->mixFuncOffset](v, bufferPosition, samplesToMix);
 		}
 
 		if (r->active) // volume ramp fadeout-voice
-			mixFuncTab[(3*5*2) + r->mixFuncOffset](r, bufferPosition, samplesToMix);
+			mixFuncTab[mixOffsetBias + r->mixFuncOffset](r, bufferPosition, samplesToMix);
 	}
 }
 
@@ -783,14 +763,14 @@ static void fillVisualsSyncBuffer(void)
 	for (int32_t i = 0; i < song.numChannels; i++, c++, s++, v++)
 	{
 		c->scopeVolume = v->scopeVolume;
-		c->scopeDelta = v->scopeDelta;
+		c->period = s->finalPeriod;
 		c->instrNum = s->instrNum;
 		c->smpNum = s->smpNum;
 		c->status = s->tmpStatus;
 		c->smpStartPos = s->smpStartPos;
 
 		c->pianoNoteNum = 255; // no piano key
-		if (songPlaying && (c->status & IS_Period) && !s->keyOff)
+		if (songPlaying && ui.instEditorShown && (c->status & IS_Period) && !s->keyOff)
 		{
 			const int32_t note = getPianoKey(s->finalPeriod, s->finetune, s->relativeNote);
 			if (note >= 0 && note <= 95)
@@ -952,13 +932,28 @@ bool setupAudio(bool showErrorMsg)
 	want.callback = audioCallback;
 	want.samples  = configAudioBufSize;
 
-	audio.dev = SDL_OpenAudioDevice(audio.currOutputDevice, 0, &want, &have, SDL_AUDIO_ALLOW_ANY_CHANGE);
+	char *device = audio.currOutputDevice;
+	if (device != NULL && strcmp(device, DEFAULT_AUDIO_DEV_STR) == 0)
+		device = NULL; // force default device
+
+	audio.dev = SDL_OpenAudioDevice(device, 0, &want, &have, SDL_AUDIO_ALLOW_FREQUENCY_CHANGE);
 	if (audio.dev == 0)
 	{
-		if (showErrorMsg)
-			showErrorMsgBox("Couldn't open audio device:\n\"%s\"\n\nDo you have an audio device enabled and plugged in?", SDL_GetError());
+		audio.dev = SDL_OpenAudioDevice(NULL, 0, &want, &have, SDL_AUDIO_ALLOW_FREQUENCY_CHANGE);
+		if (audio.currOutputDevice != NULL)
+		{
+			free(audio.currOutputDevice);
+			audio.currOutputDevice = NULL;
+		}
+		audio.currOutputDevice = strdup(DEFAULT_AUDIO_DEV_STR);
 
-		return false;
+		if (audio.dev == 0)
+		{
+			if (showErrorMsg)
+				showErrorMsgBox("Couldn't open audio device:\n\"%s\"\n\nDo you have an audio device enabled and plugged in?", SDL_GetError());
+
+			return false;
+		}
 	}
 
 	// test if the received audio format is compatible
@@ -982,6 +977,7 @@ bool setupAudio(bool showErrorMsg)
 		return false;
 	}
 
+	/*
 	if (have.freq != 44100 && have.freq != 48000 && have.freq != 96000)
 	{
 		if (showErrorMsg)
@@ -990,6 +986,7 @@ bool setupAudio(bool showErrorMsg)
 		closeAudio();
 		return false;
 	}
+	*/
 
 	if (!setupAudioBuffers())
 	{
