@@ -42,6 +42,9 @@ enum // CHECKBOXES
 	CB_INST_EXT_MIDI,
 	CB_INST_EXT_MUTE,
 
+	// SAMPLE EDITOR EFFECTS
+	CB_SAMPFX_NORMALIZATION,
+
 	// TRIM
 	CB_TRIM_PATT,
 	CB_TRIM_INST,
