@@ -12,11 +12,13 @@
 typedef struct midi_t
 {
 	char *inputDeviceName, *inputDeviceNames[MAX_MIDI_DEVICES];
-	volatile bool closeMidiOnExit, initThreadDone;
-	uint32_t inputDevice;
-	bool enable, rescanDevicesFlag;
+	volatile bool initThreadDone, callbackBusy, enable;
+	bool rescanDevicesFlag;
+	uint32_t inputDevice, numInputDevices;
 	int16_t currMIDIVibDepth, currMIDIPitch;
-	int32_t numInputDevices;
+#ifdef __APPLE__
+	SDL_Thread *initMidiThread;
+#endif
 } midi_t;
 
 extern midi_t midi; // ft2_midi.c
@@ -35,6 +37,6 @@ void scrollMidiInputDevListUp(void);
 void scrollMidiInputDevListDown(void);
 void sbMidiInputSetPos(uint32_t pos);
 bool testMidiInputDeviceListMouseDown(void);
-int32_t SDLCALL initMidiFunc(void *ptr);
+int32_t initMidiFunc(void *ptr);
 
 #endif
