@@ -20,6 +20,9 @@
 bool loadFLAC(FILE *f, uint32_t filesize);
 #endif
 
+bool detectBRR(FILE *f);
+bool loadBRR(FILE *f, uint32_t filesize);
+
 bool loadAIFF(FILE *f, uint32_t filesize);
 bool loadIFF(FILE *f, uint32_t filesize);
 bool loadRAW(FILE *f, uint32_t filesize);
@@ -31,14 +34,15 @@ enum
 	FORMAT_IFF = 1,
 	FORMAT_WAV = 2,
 	FORMAT_AIFF = 3,
-	FORMAT_FLAC = 4
+	FORMAT_FLAC = 4,
+	FORMAT_BRR = 5
 };
 
 // file extensions accepted by Disk Op. in sample mode
 char *supportedSmpExtensions[] =
 {
 	"iff", "raw", "wav", "snd", "smp", "sam", "aif", "pat",
-	"aiff","flac", // IMPORTANT: Remember comma after last entry!!!
+	"aiff","flac","brr", // IMPORTANT: Remember comma after last entry!!!
 
 	"END_OF_LIST" // do NOT move, remove or edit this line!
 };
@@ -78,6 +82,9 @@ static int8_t detectSample(FILE *f)
 	if (!memcmp("FORM", &D[0], 4) && (!memcmp("AIFF", &D[8], 4) || !memcmp("AIFC", &D[8], 4)))
 		return FORMAT_AIFF;
 
+	if (detectBRR(f))
+		return FORMAT_BRR;
+
 	return FORMAT_UNKNOWN;
 }
 
@@ -125,6 +132,7 @@ static int32_t SDLCALL loadSampleThread(void *ptr)
 		case FORMAT_IFF: sampleLoaded = loadIFF(f, filesize); break;
 		case FORMAT_WAV: sampleLoaded = loadWAV(f, filesize); break;
 		case FORMAT_AIFF: sampleLoaded = loadAIFF(f, filesize); break;
+		case FORMAT_BRR: sampleLoaded = loadBRR(f, filesize); break;
 		default: sampleLoaded = loadRAW(f, filesize); break;
 	}
 	fclose(f);
@@ -136,7 +144,7 @@ static int32_t SDLCALL loadSampleThread(void *ptr)
 
 	if (!smpFilenameSet) // if we didn't set a custom sample name in the loader, set it to its filename
 	{
-		char *tmpFilename = unicharToCp437(editor.tmpFilenameU, true);
+		char *tmpFilename = unicharToCp850(editor.tmpFilenameU, true);
 		if (tmpFilename != NULL)
 		{
 			int32_t i = (int32_t)strlen(tmpFilename);
