@@ -1,6 +1,8 @@
 #pragma once
 
 #include "../ft2_audio.h"
+#include "ft2_quadratic_spline.h"
+#include "ft2_cubic_spline.h"
 #include "ft2_windowed_sinc.h"
 
 /* ----------------------------------------------------------------------- */
@@ -78,8 +80,15 @@
 	fVolumeL += fVolumeLDelta; \
 	fVolumeR += fVolumeRDelta;
 
+/* It may look like we are potentially going out of bounds while looking up the sample points,
+** but the sample data is actually padded on both the left (negative) and right side, where correct tap
+** samples are stored according to loop mode (or no loop).
+**
+** There is also a second special case for the left edge (negative taps) after the sample has looped once.
+*/
+
 /* ----------------------------------------------------------------------- */
-/*                            NO INTERPOLATION                             */
+/*                  NO INTERPOLATION (NEAREST NEIGHBOR)                    */
 /* ----------------------------------------------------------------------- */
 
 #define RENDER_8BIT_SMP \
@@ -92,19 +101,16 @@
 	*fMixBufferL++ += fSample * fVolumeL; \
 	*fMixBufferR++ += fSample * fVolumeR;
 
+
 /* ----------------------------------------------------------------------- */
 /*                          LINEAR INTERPOLATION                           */
 /* ----------------------------------------------------------------------- */
 
-/* It may look like we are potentially going out of bounds while looking up the sample points,
-** but the sample data has a fixed sample after the end (sampleEnd/loopEnd).
-*/
-
 #define LINEAR_INTERPOLATION(s, f, scale) \
 { \
 	const int32_t frac = (uint32_t)(f) >> 1; /* uint32 -> int32 range, faster int->float conv. (x86/x86_64) */ \
-	const float fFrac = frac * (1.0f / (MIXER_FRAC_SCALE/2)); /* 0.0f .. 0.9999999f */ \
-	fSample = ((s[0] + (s[1]-s[0]) * fFrac)) * (1.0f / scale); \
+	const float fFrac = (int32_t)frac * (1.0f / (MIXER_FRAC_SCALE/2)); /* 0.0f .. 0.9999999f */ \
+	fSample = (s[0] + ((s[1] - s[0]) * fFrac)) * (1.0f / scale); \
 }
 
 #define RENDER_8BIT_SMP_LINTRP \
@@ -117,22 +123,41 @@
 	*fMixBufferL++ += fSample * fVolumeL; \
 	*fMixBufferR++ += fSample * fVolumeR;
 
+
+/* ----------------------------------------------------------------------- */
+/*                     QUADRATIC SPLINE INTERPOLATION                      */
+/* ----------------------------------------------------------------------- */
+
+// through LUT: mixer/ft2_quadratic_spline.c
+
+#define QUADRATIC_SPLINE_INTERPOLATION(s, f, scale) \
+{ \
+	const float *t = fQuadraticSplineLUT + (((uint32_t)(f) >> QUADRATIC_SPLINE_FRACSHIFT) * QUADRATIC_SPLINE_WIDTH); \
+	fSample = ((s[0] * t[0]) + \
+	           (s[1] * t[1]) + \
+	           (s[2] * t[2])) * (1.0f / scale); \
+}
+
+#define RENDER_8BIT_SMP_QINTRP \
+	QUADRATIC_SPLINE_INTERPOLATION(smpPtr, positionFrac, 128) \
+	*fMixBufferL++ += fSample * fVolumeL; \
+	*fMixBufferR++ += fSample * fVolumeR;
+
+#define RENDER_16BIT_SMP_QINTRP \
+	QUADRATIC_SPLINE_INTERPOLATION(smpPtr, positionFrac, 32768) \
+	*fMixBufferL++ += fSample * fVolumeL; \
+	*fMixBufferR++ += fSample * fVolumeR;
+
+
 /* ----------------------------------------------------------------------- */
 /*                       CUBIC SPLINE INTERPOLATION                        */
 /* ----------------------------------------------------------------------- */
 
 // through LUT: mixer/ft2_cubic_spline.c
 
-/* It may look like we are potentially going out of bounds while looking up the sample points,
-** but the sample data is actually padded on both the left (negative) and right side, where correct tap
-** samples are stored according to loop mode (or no loop).
-**
-** There is also a second special case for the left edge (negative taps) after the sample has looped once.
-*/
-
 #define CUBIC_SPLINE_INTERPOLATION(s, f, scale) \
 { \
-	const float *t = fCubicSplineLUT + (((uint32_t)(f) >> CUBIC_SPLINE_FSHIFT) & CUBIC_SPLINE_FMASK); \
+	const float *t = fCubicSplineLUT + (((uint32_t)(f) >> CUBIC_SPLINE_FRACSHIFT) & CUBIC_SPLINE_FRACMASK); \
 	fSample = ((s[-1] * t[0]) + \
 	           ( s[0] * t[1]) + \
 	           ( s[1] * t[2]) + \
@@ -166,22 +191,16 @@
 	*fMixBufferL++ += fSample * fVolumeL; \
 	*fMixBufferR++ += fSample * fVolumeR;
 
+
 /* ----------------------------------------------------------------------- */
 /*                       WINDOWED-SINC INTERPOLATION                       */
 /* ----------------------------------------------------------------------- */
 
 // through LUTs: mixer/ft2_windowed_sinc.c
 
-/* It may look like we are potentially going out of bounds while looking up the sample points,
-** but the sample data is actually padded on both the left (negative) and right side, where correct tap
-** samples are stored according to loop mode (or no loop).
-**
-** There is also a second special case for the left edge (negative taps) after the sample has looped once.
-*/
-
 #define WINDOWED_SINC8_INTERPOLATION(s, f, scale) \
 { \
-	const float *t = v->fSincLUT + (((uint32_t)(f) >> SINC8_FSHIFT) & SINC8_FMASK); \
+	const float *t = v->fSincLUT + (((uint32_t)(f) >> SINC1_FRACSHIFT) & SINC1_FRACMASK); \
 	fSample = ((s[-3] * t[0]) + \
 	           (s[-2] * t[1]) + \
 	           (s[-1] * t[2]) + \
@@ -194,7 +213,7 @@
 
 #define WINDOWED_SINC16_INTERPOLATION(s, f, scale) \
 { \
-	const float *t = v->fSincLUT + (((uint32_t)(f) >> SINC16_FSHIFT) & SINC16_FMASK); \
+	const float *t = v->fSincLUT + (((uint32_t)(f) >> SINC2_FRACSHIFT) & SINC2_FRACMASK); \
 	fSample = (( s[-7] * t[0]) + \
 	           ( s[-6] * t[1]) + \
 	           ( s[-5] * t[2]) + \
@@ -261,6 +280,7 @@
 	*fMixBufferL++ += fSample * fVolumeL; \
 	*fMixBufferR++ += fSample * fVolumeR;
 
+
 /* ----------------------------------------------------------------------- */
 /*                      SAMPLES-TO-MIX LIMITING MACROS                     */
 /* ----------------------------------------------------------------------- */
